local cql = ClassicQuestLog

--[[ BindButton overrides the TOGGLEQUESTLOG keybinding ]]

cql.BindButton = ClassicQuestLogBindButton

function cql.BindButton:Setup()
    if cql.settings.ShowFromKeyBind then -- overriding bind, watch for bind changes
        self:RegisterEvent("UPDATE_BINDINGS")
    else -- not overriding bind, can stop watching for bind changes
        self:UnregisterEvent("UPDATE_BINDINGS")
    end
    self:Update() -- it's ok if in combat, it will try again when leaving
end

function cql.BindButton:OnEvent(event,...)
    if self[event] then
        self[event](self,event,...)
    end
end

-- UPDATE_BINDINGS can commonly trigger in sets, and may nil the bind in one event and set it in another.
-- this will wait 0.5 seconds after a set of UPDATE_BINDINGS to run UpdateBind
function cql.BindButton:UPDATE_BINDINGS(...)
    cql.timer:Start(0.5,cql.BindButton.DelayedUpdate)
end

-- only registered when a bind changed during combat
function cql.BindButton:PLAYER_REGEN_ENABLED()
    self:UnregisterEvent("PLAYER_REGEN_ENABLED")
    self:Update()
end

function cql.BindButton:Update()
    if InCombatLockdown() then
        self:RegisterEvent("PLAYER_REGEN_ENABLED")
    elseif cql.settings.ShowFromKeyBind then -- if override enabled, rebind if changed
        local key = GetBindingKey("TOGGLEQUESTLOG")
        if self.key ~= key then
            ClearOverrideBindings(self)
            if key then
                SetOverrideBindingClick(self,true,key,"ClassicQuestLogBindButton","LeftButton")
            end
            self.key = key
        end
    else -- if override not enabled, clear bind
        ClearOverrideBindings(self)
        self.key = nil
    end
end

function cql.BindButton:DelayedUpdate()
    cql.BindButton:Update()
end

--[[ MicroButton overrides the quest log button in the default micro button menu ]]

cql.MicroButton = ClassicQuestLogMicroButton

-- micro button override is with a new button placed in a higher framelevel than default
-- and the default alpha set to 0 (setting alpha instead of hiding to minimize taint possibility)
function cql.MicroButton:Setup()
    local parent = self:GetParent()
    parent:HookScript("OnEnable",function()
        self:Enable()
        self.Background:SetAlpha(1)
        self:Update()
    end)
    parent:HookScript("OnDisable",function()
        self:Disable()
        self.Background:SetAlpha(0.5)
        self:Update()
    end)
    self:Update()
end

-- if ShowFromMicroButton enabled, then show this addon's micro button above default
function cql.MicroButton:Update()
    local parent = self:GetParent()
    if ClassicQuestLog.settings.ShowFromMicroButton then
        self:Show()
        parent:SetAlpha(0)
        self:SetFrameLevel(parent:GetFrameLevel()+2)
    else
        self:Hide()
        parent:SetAlpha(1)
    end
end

function cql.MicroButton:OnEnter()
    GameTooltip:SetOwner(self,"ANCHOR_RIGHT")
    local bindingKey = GetBindingKey("TOGGLEQUESTLOG")
    GameTooltip:AddLine("\124cffffffffClassic Quest Log\124r"..(bindingKey and " ("..bindingKey..")" or ""))
    GameTooltip:Show()
end

function cql.MicroButton:OnLeave()
    GameTooltip:Hide()
end

function cql.MicroButton:OnClick()
    cql:Toggle()
end

--[[ ObjectiveTracker is not a frame but methods to handle overriding objective tracker clicks ]]

cql.ObjectiveTracker = {}

function cql.ObjectiveTracker:Setup()
    if IsAddOnLoaded("Blizzard_ObjectiveTracker") then
        -- one-time hooks of quest objective to summon classic quest log instead of world map
        if not self.isHooked then
            hooksecurefunc(QUEST_TRACKER_MODULE,"OnBlockHeaderClick",self.OnClick)
            hooksecurefunc(CAMPAIGN_QUEST_TRACKER_MODULE,"OnBlockHeaderClick",self.OnClick)
            self.isHooked = true
        end
    else -- objective tracker wasn't loaded on login, wait until it loads
        cql:RegisterEvent("ADDON_LOADED")
    end
end

-- clicking of a quest or campaign on objective tracker
function cql.ObjectiveTracker:OnClick(block,button)
    if not cql.settings.ShowFromObjectiveTracker then
        return -- 'Override Objective Tracker' is disabled
    end
    if IsModifiedClick("CHATLINK") and ChatEdit_GetActiveWindow() then
        return -- user was linking quest to chat
    end
    if InCombatLockdown() then
        return -- user is in combat
    end
    if IsModifiedClick("QUESTWATCHTOGGLE") then
        return -- user was untracking a quest
    end
    if button=="RightButton" then
        return -- use right-clicked quest, let default do that stuff
    end
    ClassicQuestLog.Log:Update()
    -- the selected questID should've been selected in the act of clicking an objective.
    -- if the questID is valid, close the map that default UI showed and scroll to the questID
    local questID = cql.Log:GetSelectedQuest()
    if C_QuestLog.IsOnQuest(questID) then
        if not cql:IsVisible() then
            cql:Toggle()
        end
        cql:SetMode("Detail")
        ToggleWorldMap()
        if WorldMapFrame:IsVisible() then
            ToggleWorldMap()
        end
        cql:Update()
        cql.timer:Start(0.1,cql.Log.DelayedScrollToSelectedQuestID)
    end
end
